#include "SeparatrixValueComputation.h"
#include "C0SetCartesianProduct.h"
#include "plots.h"

using namespace capd::matrixAlgorithms;

////////////////////////////////////////
// function: backwardConvergenceBound()
//
// Purpose:
//    Consider an ODE 
//        gamma' = (A + B(t))*gamma
//    where
//           (0.0  0.0  0.0)
//        A= (0.0 -a22  0.0)
//           (0.0  0.0 -a33)
//    and B(t) = (bij(t)) with the following bound for all t<0
//        |bij(t)| < c*exp(-lambda*|t|)
//    The function backwardConvergenceBound() computes a set
//         U = [1-rho,1+rho] x [-rho,rho] x [-rho,rho]
//    for which we know that gamma(t) converges to a point {x} x {0} x {0} in U for t->-\infty 
IVector backwardConvergenceBound(const interval &c, const interval &lambda, const interval &a22,const interval &a33)
{
	if(! ((min(a22,a33)-6.0*c)>0.0)) abort("Error: unable to compute the limitConvergenceBound. Aborting program. 1");
	if(! (4*c<lambda) ) abort("Error: unable to compute the limitConvergenceBound. Aborting program. 2");
	interval delta=1.001;
	interval rho = (delta*max(c/(lambda-4.0*c),2.0*c/(min(a22,a33)-6.0*c))).right();
	IVector U(3);
	U[0] = 1.0 + intervalHull(-rho,rho);
	U[1] = intervalHull(-rho,rho);
	U[2] = intervalHull(-rho,rho);
	return U;
}

////////////////////////////////////////
// function: forwardConvergenceBound()
//
// Purpose:
//    Consider an ODE 
//        gamma' = (A + B(t))*gamma
//    where
//           (0.0  0.0  0.0)
//        A= (0.0 -a22  0.0)
//           (0.0  0.0 -a33)
//    and B(t) = (bij(t)) with the following bound for all t>0
//        |bij(t)| < c*exp(-lambda*|t|)
//    The function forwardConvergenceBound() computes a set
//         U = [x0-rho,x0+rho] x [-R,R] x [-R,R]
//    for which we know that gamma(t) converges to a point {x} x {0} x {0} in U for t->+\infty 
// Remark: The differences between the previous backwardConvergenceBound() are: 
//    - we start from a set in which we have radius R on the two last coordinates of U instead of rho. 
//    - We have x0 on the first coordinate of U.
//    - the limit is as t-> +infinity  (and not -infinity)
IVector forwardConvergenceBound(const interval &c, const interval &lambda, const interval &a22,const interval &a33,
								const interval &x0, const interval &R)
{
	if(! (0<lambda-2*c) ) abort("Error: unable to compute the forwardConvergenceBound. Aborting program. 1");
	if(! (0<min(a22,a33)-6*c) ) abort("Error: unable to compute the forwardConvergenceBound. Aborting program. 2");
	interval delta=1.001;
	interval rho=(delta*(c*(x0+2.0*R)/(lambda-2.0*c))).right();

	if(! (rho<0.5*R*(min(a22,a33)/(2.0*c)-1.0)-x0*0.5) ) abort("Error: unable to compute the forwardConvergenceBound. Aborting program. 3");
	IVector U(3);
	U[0] = x0 + intervalHull(-rho,rho);
	U[1] = intervalHull(-R,R);
	U[2] = intervalHull(-R,R);
	return U;
}

interval SeparatrixValue(interval a, interval c,interval xi,interval ru, WuEnclosure wu1d, WuEnclosure ws2d,interval returnTime)
{
	capd::vectalg::EuclNorm <IVector,IMatrix> euclNorm;
	
	string vField="par:a;var:x,y,z,h1,h2,h3;fun:y,(a+1.0)*(1.0-z)*x-a*y,-z+x^2,-(a+1.0)*h1-(a+1.0)*(1.0-z)*h2-2*x*h3,-h1-h2,(a+1.0)*x*h2-a*h3;";
	c=((1.0+a)*c).right();
	
	// linear coortinate change for the coordinate gamma:
	IMatrix P(3,3);
	P[0][0]=1.0;  P[0][1]=a+1.0; P[0][2]=0.0;
	P[1][0]=-1.0;   P[1][1]=1.0;   P[1][2]=0.0;
	P[2][0]=0.0;   P[2][1]=0.0;   P[2][2]=1.0;
	interval a22=a+2.0;
	interval a33=a;
	
	// linear coordinate changes around the fixed point in coordinates (x,y,z) 
	IMatrix C1=wu1d.f->C;
	IMatrix C2=ws2d.f->C;
	
	IVector xu(1);
	xu[0]=ru;
	IVector U1 = backwardConvergenceBound(c*euclNorm(C1*wu1d.localImage(xu)),xi,a22,a33);
	
	// The unstable manifold, in coordinates X,Y,Z of the system is inside of the set
	//   x0 + C1*bu
	// we now compute the x0 and bu:
	IVector bu=wu1d.localImage(xu);
	bu=bu-midVector(bu);
	IVector x0=C1*midVector(wu1d.localImage(xu));
	
	// The initial point \eta is
	//   P*U1 = P*midVector(U1) + P*(U1-midVector(U1))
	IVector X0=P*midVector(U1);
	IVector Bu=U1-midVector(U1);
	
	C0Rect2Set Yu=diagonalC0Rect2Set(x0,X0,C1,P,bu,Bu); // This is the initial point for the integration
	
	IMap f(vField);
	f.setParameter("a",a);  
	
	int order=15;
	ITaylor solver(f,order);
	ITimeMap phi(solver);
	
	phi(returnTime,Yu);
		
	IVector Q=Yu;
	IVector xu2(3);
	for(int i=0;i<3;i++) xu2[i]=Q[i];
	
	IVector Xu2(3);
	for(int i=0;i<3;i++) Xu2[i]=Q[i+3];
	IVector U=gauss(P,Xu2);
	interval R = sqrt((U[1]*U[1]).right()+(U[2]*U[2]).right());	
	IVector U2 = forwardConvergenceBound(c*euclNorm(gauss(C2,xu2)),xi,a22,a33,U[0],R);
	
	C0Rect2Set Y=diagonalC0Rect2Set(x0,X0,C1,P,bu,Bu);
	
	plotSeparatrixValueTrajectory(P,Y,solver,returnTime,0.1,"pic/sepValTraj","pic/sepValTrajOriginalCoord");
	cout << "U1 = " << U1 << endl;
	cout << "U2 = " << U2 << endl;
	return U2[0]/U1[0];
	
}















