#include "capd/capdlib.h"
#include "utils.h"
#include "LocalVField.h"
#include "WuEnclosure.h"
#include "ratecond.h"
#include "eigenvector.h"
#include "plots.h"
#include "SeparatrixValueComputation.h"
#include <iostream>

using namespace std;
using namespace capd;
using namespace capd::matrixAlgorithms;
using namespace capd::vectalg;

//////////////////////////////////////////////////////////////////////

//////////////////////////////
// Function: computeEigenvalues()
//
// Purpose: This function computes and displays eigenvalues of the Jacobian of a
// vector field f, at a point x.
// The aim is simply to display these values in order to verify hypebolicity. They are not used
// anywhere else than that in the program.
//
// Method: We first compute an initial guess on the eigenvalue and eigenvector nonrigorously.
// This is then converted to a set enclosure and validated using:
//   containsEigenvalueAndEigenvector()
// from eigenvector.h/.cpp
void computeEigenvalues(IMap &f,IVector x)
{
	// ---------------------------
	// computation of the candidates for eigenvalues and eigenvectors:
	IMatrix A=f[x];
	DMatrix B(3,3);
	for(int i=0;i<3;i++)
		for(int k=0;k<3;k++) B[i][k]=A[i][k].mid().leftBound();
  	DVector rE(3), iE(3);         	// real and imaginary parts of eigenvalues
  	DMatrix rVec(3,3), iVec(3,3); 	// real and imaginary parts of eigenvectors
  	computeEigenvaluesAndEigenvectors(B,rE,iE,rVec,iVec);
	// end of computation of the candidates for eigenvalues and eigenvectors.
	// ---------------------------
	
    IVector xRe(3),xIm(3);
	double r=pow(10.0,-6.0); // this is the factor by which we "blow up" the initial guess
	
	for(int k=0;k<3;k++) 
	{
		for(int i=0;i<3;i++) xRe[i] = rVec[i][k]+r*interval(-1.0,1.0); // blowing up the initial guess
		for(int i=0;i<3;i++) xIm[i] = iVec[i][k]+r*interval(-1.0,1.0); // blowing up the initial guess
    
	    interval rho=rE[k]+r*interval(-1.0,1.0); // blowing up the initial guess
	    interval omega=iE[k]+r*interval(-1.0,1.0); // blowing up the initial guess
		
		
		if(abs(omega) < 2.0*r) // this means that it is likely that the eigenvalue is real, hence below we check for a real eigenvalue:
		{
			// If containsEigenvalueAndEigenvector(A,rho,xRe) returns 1, then this ensures that
			// the eigenvector and eigenvalue is real:
			if(containsEigenvalueAndEigenvector(A,rho,xRe)==0) abort("Problem with computing eigenvalues. Aborting program.");
			cout << "Eigenvalue " << k+1 << " Re = " << rho << "    Im = 0.0 " << endl;
		}
		else // we check for a complex eigenvalue:
		{
			if(containsEigenvalueAndEigenvector(A,rho,omega,xRe,xIm)==0) abort("Problem with computing eigenvalues. Aborting program.");
			cout << "Eigenvalue " << k+1 << " Re = " << rho << "    Im = " << omega << endl;
		}
	}
	
}

void ShimizuMorioka()
{
	IMatrix C1(3,3);
	C1[0][0]=1.0;  C1[0][1]=-0.36706363121968; C1[0][2]=0.0;
	C1[1][0]=1.0;  C1[1][1]=1.0;               C1[1][2]=0.0;
	C1[2][0]=0.0;  C1[2][1]=0.0;               C1[2][2]=1.1;
	IMatrix C2=C1;
	swapColumns(C2,0,2);
	
	string vFieldWu1="par:a;var:x,y,z;fun:y,(a+1.0)*(1.0-z)*x-a*y,-z+x^2;";
	string vFieldWu2="par:a;var:x,y,z;fun:-y,-(a+1.0)*(1.0-z)*x+a*y,z-x^2;";
	double a0=1.72432329151541;
	interval a=a0+interval(-1.0,1.0)*pow(10.0,-13.0);
	IVector x0(3);
	
	IMap f1(vFieldWu1);  f1.setParameter("a",a);
	IMap f2(vFieldWu2);  f2.setParameter("a",a);
	
	// We now verify that all the fixed points are hyperbolic
	computeEigenvalues(f1,x0);
	
	// Our domain in which we enclose the manifolds is D = Bu(R) x Bs(R)
	// Below is the radius R
	interval R=pow(10.0,-5.0);
	
	// -------------------------------
	// Here we establish the 2-dimensional stable manifold 
	// (which is here treated as an unstable manifold for reversed sign vector field)
	
	// The manifold is considered in this local vector field
	LocalVField h2(f2,x0,C2);
	
	// This is the two dimensional domain of the stable manifold Ws. We choose so that it contains 
	// the two dimensional ball Bs(R).
	IVector domain2(2);
	domain2[0]=R*interval(-1.0,1.0);
	domain2[1]=R*interval(-1.0,1.0);
	
	// This is the Lipschitz constant for Ws
	interval L2=2.0*pow(10.0,-5.0);
	
	// The domain in which we establish Ws is
	//   D = domain2 x Bs
	// Below we choose Bs so that it contains a ball (in fact one dimensional interval) or radius R.
	IVector Bs(1);
	Bs[0]=domain2[0];
	
	// Finally we establish Ws and plot it. While the constructor is initiated, all needed assumptions are checked.
	// if some of them are not satisfied, the constructor will terminate the program and throw an error.
	WuEnclosure wu2(h2,domain2,L2,Bs);
	interval xi2=wu2.xi.left();
	plotWs(wu2);
	
	// We will divide the parameter range E into N intervals and consider each interval separately.
	// For each of the N subintervals we condust the followng steps:
	// 1. compute local enclosure of the unstable manifold inside of D. This is done in local coordinates.
	// 2. Propagate a point 'pu' from the manifold along the flow for time T computing Phi(T,pu). 
	//    Both 'pu' and 'p' are in the original (not local) coordinates
	// 3. transport p to local coordinates, and check that it is in D
	// 4. Compute h(G) for the lowest and highest considered parameters.
	
	ofstream fileWU;
	fileWU.open("pic/WU.txt");
	
	int N=5;
	//interval derivativeBound;
	interval T=26;
	IVector xu(1);
	xu[0]=R;
	IVector domain1(1);
	domain1[0]=R*interval(-1.0,1.0);
	interval L1=2*pow(10.0,-5.0);
	interval xi1;
	
	for(int k=0;k<N+2;k++)
	{
		if(k<N) f1.setParameter("a",part(a,N,k));
		if(k==N) f1.setParameter("a",a.left());
		if(k==N+1) f1.setParameter("a",a.right());
			
		// 1.
		LocalVField h1(f1,x0,C1);
		WuEnclosure wu1(h1,domain1,L1);
		if(k==0) xi1=wu1.xi;
		if(k>0) xi1=intervalHull(xi1,wu1.xi).left();
		
		// 2.
		IOdeSolver solver(f1,25);
		ITimeMap Phi(solver);
	
		C0Rect2Set pu=wu1(xu);
		
		IVector p=Phi(T,pu);

		if(k==N) plotWu(p,C1,x0,"pic/WuLeft.txt");
		if(k==N) plotWu(wu1,"pic/WuGLeft.txt");
		if(k==N+1) plotWu(p,C1,x0,"pic/WuRight.txt");
		if(k==N+1) plotWu(wu1,"pic/WuGRight.txt");
		if(k<N) plot(fileWU,gauss(C1,p-x0)[0],gauss(C1,p-x0)[1]);
		
		// 3.
		p=gauss(C2,p-x0);
		IVector y(2);
		y[0]=p[0];
		y[1]=p[1];
		if(! (p.euclNorm()<R)) abort("Phi(T,pu) is not in D! Aborting program.");
		if(! subsetInterior(p[2],Bs[0])) abort("Phi(T,pu) is not in D! Aborting program.");			
		
		// 4.
		if(k<N) cout << "verified that PhiT(pu,G_" << k << ") is in D " << endl;
		if(k==N) cout << "h("<< a.left() << ") = " << (p-wu2.localImage(y))[2] <<  endl;
		if(k==N+1) cout << "h(" << a.right() <<") = " << (p-wu2.localImage(y))[2] <<  endl;	
	}
	
	cout << "xi1 = " << xi1 << endl;
	cout << "xi2 = " << xi2 << endl;
	EuclNorm <IVector,IMatrix> euclNorm;
	
	
	interval c=3.5;
	interval xi=1.0-power(interval(10.0),-4);
	if(!(2.0*sqrt(1.0+power(L2,2))*euclNorm(C1)*euclNorm(gaussInverseMatrix(C1))<3.5)) abort("problem with expansion bound. Aborting program!");
	if(!(xi<xi1)) abort("problem with expansion bound. Aborting program!");
	if(!(xi<xi2)) abort("problem with expansion bound. Aborting program!");
	
	// The bound c for the coefficients of the matrix B(t) used for the computation of the separatrix value:
	
	f1.setParameter("a",a);
	LocalVField h1(f1,x0,C1);
	WuEnclosure wu1(h1,domain1,L1);
	interval A=SeparatrixValue(a,c,xi,R,wu1,wu2,T);
	cout << "sep val = " << A << endl;
	
	// to skip making plots one coan comment out the below "return;"
	//return;
	
	// making plots (which are not needed for the proof, but serve only as an ilustration):
	plotWuTrajectory(f1,wu1,xu[0],T,0.1,"pic/Wu_trajectory_all.txt");
	plotWuInOriginalCoord(wu1,"pic/WuInOriginalCoordinates.txt");
	
	f1.setParameter("a",a.right());
	h1=LocalVField(f1,x0,C1);
	wu1=WuEnclosure(h1,domain1,L1);
	plotWuTrajectory(f1,wu1,xu[0],T,0.1,"pic/Wu_trajectory_right.txt");
	
	f1.setParameter("a",a.left());
	h1=LocalVField(f1,x0,C1);
	wu1=WuEnclosure(h1,domain1,L1);
	plotWuTrajectory(f1,wu1,xu[0],T,0.1,"pic/Wu_trajectory_left.txt");
	
	plotWsInOriginalCoord(wu2,"pic/WsInOriginalCoordinates.txt");
	
}

int main(int argc, char* argv[])
{
    time_t before,after;
    before=time(NULL);
	
  	cout.precision(14);
	try
	{
		ShimizuMorioka();
	}
	catch(exception& e)
  	{
    		cout << "\n\nException caught: "<< e.what() << endl;
  	}
    
	after=time(NULL);
    cout << "time: " << difftime(after,before) << " s. " << endl;
  	return 0;
} 
