#include "plots.h"

////////////////////////
// Function: plotWs()
//
// Purpose: Plotting the enclosure of the two dimensional stable manifold.
// Note: In the proof, all manifolds are of the form (x,w(x)). We are reversing the 
// roles of the coordinates, since we want to plot (w(y),y).
void plotWs(WuEnclosure &wu2)
{
	ofstream file;
	file.open("pic/Ws.txt");
	int n=100;
	for(int i=0;i<n;i++) 
	{
		for(int k=0;k<n;k++) 
		{
			IVector x(2);
			x[0]=part(wu2.domain[0]/1.01,n,i);
			x[1]=part(wu2.domain[1]/1.01,n,k);
			IVector p=wu2.localImage(x);
			plot(file,p[2],p[1]); // here x is on coordinate [2] since coordinates of Ws are interchanged with respect to Wu
		}
	}
}

///////////////////////
// Function: plotWu()
//
// Purpose: Plotting a point in the local coordinates of the 1-dimensional unstable manifold.
// The point p will be chosen to be Phi(T,(x,wu(x))). In other words, the point p will be the point
// from the unstable manifold, after it returns to the domein D.
void plotWu(IVector p,IMatrix C1,IVector x,string fileName)
{
	ofstream file;
	file.open(fileName);
	p=gauss(C1,p-x);
	plot(file,p[0],p[1]);
}

////////////////////////
// Function: plotWu()
//
// Purpose: Plotting the enclosure of the one dimensional unstable manifold, in its local coordinates.
void plotWu(WuEnclosure &wu1,string fileName)
{
	ofstream file;
	file.open(fileName);
	int n=100;
	for(int i=0;i<n;i++) 
	{
		IVector x(1);
		x[0]=part(wu1.domain[0]/1.01,n,i);
		IVector p=wu1.localImage(x);
		plot(file,p[0],p[1]);
	}
}

///////////////////////
// Function: plotWu()
//
// Purpose: This function plots the trajectory, which starts from wu(x) and ends at Phi(T,wu(x)) 
void plotWuTrajectory(IMap &f1,WuEnclosure &wu,interval x,interval T,double timeStep,string fileName)
{
	ofstream file;
	file.open(fileName);
	cout.precision(12);
	IOdeSolver solver(f1,20);
	solver.setStep(timeStep);
	ITimeMap timeMap(solver);
		
	// This is our initial condition
	IVector v(1);
	v[0]=x;
	C0Rect2Set s=wu(v);
	timeMap.stopAfterStep(true);
	interval prevTime(0.);
	do 
	{
	   timeMap(T,s);
	   interval stepMade = solver.getStep();
	   const IOdeSolver::SolutionCurve& curve = solver.getCurve();
	   interval domain = interval(0,1)*stepMade;
   
	   int grid=100;
	   for(int i=0;i<grid;++i)
	   {
		   interval subsetOfDomain = interval(i,i+1)*stepMade/grid;
	       intersection(domain,subsetOfDomain,subsetOfDomain);
	       IVector v = curve(subsetOfDomain);
	       plot(file,v[0],v[1]);
	   }
	   prevTime = timeMap.getCurrentTime();
	}while(!timeMap.completed());
	file.close();
}

////////////////////////
// Function: plotWsInOriginalCoord()
//
// Purpose: Plotting the enclosure of the two dimensional stable manifold.
// The plot is projected onto the (x,y1) coordinates.
void plotWsInOriginalCoord(WuEnclosure &wu2,string fileName)
{
	ofstream file;
	file.open(fileName);
	int n=500;
	for(int i=0;i<n;i++) 
	{
		for(int k=0;k<n;k++) 
		{
			IVector x(2);
			x[0]=part(wu2.domain[0]/10.0,n,i); // we focus only on the fragment that is close to the fixed point. This is why we divide by 10.0
			x[1]=part(wu2.domain[1]/10.0,n,k);
		    IVector p=wu2(x);
			plot(file,p[0],p[1]);
		}
	}
}

////////////////////////
// Function: plotWuInOriginalCoord()
//
// Purpose: Plotting the enclosure of the one dimensional unstable manifold.
// The plot is projected onto the (x,y1) coordinates. 
// The plot results from local enclosure of the manifold as (x,wu(x)), which is then considered in local coordinates
void  plotWuInOriginalCoord(WuEnclosure &wu1,string fileName)
{
	ofstream file;
	file.open(fileName);
	int n=500;
	for(int i=0;i<n;i++) 
	{
		IVector x(1);
		x[0]=part(wu1.domain[0]/1000.0,n,i); // we focus only on the fragment that is close to the fixed point. This is why we divide by 1000.0
		IVector p=wu1(x);
		plot(file,p[0],p[1]);
	}
}

void plotSeparatrixValueTrajectory(IMatrix P,C0Rect2Set &Yu,ITaylor &solver,interval T,double timeStep,string fileNameGamma,string fileNameEta)
{
	ofstream file01;
	file01.open(fileNameGamma+"01.txt");
	ofstream file02;
	file02.open(fileNameGamma+"02.txt");
	
	ofstream File01;
	File01.open(fileNameEta+"01.txt");
	ofstream File02;
	File02.open(fileNameEta+"02.txt");
	
	solver.setStep(timeStep);
	ITimeMap timeMap(solver);
	
	timeMap.stopAfterStep(true);
	interval prevTime(0.);
	do 
	{
	   timeMap(T,Yu);
	   interval stepMade = solver.getStep();
	   const IOdeSolver::SolutionCurve& curve = solver.getCurve();
	   interval domain = interval(0,1)*stepMade;
   
	   int grid=100;
	   for(int i=0;i<grid;++i)
	   {
		   interval subsetOfDomain = interval(i,i+1)*stepMade/grid;
	       intersection(domain,subsetOfDomain,subsetOfDomain);
	       IVector v = curve(subsetOfDomain);
		   IVector w(3);
		   for(int k=0;k<3;k++) w[k]=v[k+3];
	       plot(File01,w[0],w[1]);
		   plot(File02,w[0],w[2]);
		   w=gauss(P,w);
	       plot(file01,w[0],w[1]);
		   plot(file02,w[0],w[2]);
	   }
	   prevTime = timeMap.getCurrentTime();
	}while(!timeMap.completed());
	file01.close();
	file02.close();
	File01.close();
	File02.close();
}
